@@ -9,6 +9,7 @@ from einops import rearrange, reduce, repeat
 from einops.layers.torch import Rearrange, Reduce
 
 from contextlib import contextmanager
+
 from enformer_pytorch import Enformer
 from enformer_pytorch.enformer_pytorch import poisson_loss, pearson_corr_coef
 from enformer_pytorch.finetune import freeze_batchnorms_, freeze_all_but_layernorms_
@@ -19,6 +20,8 @@ from tf_bind_transformer.cache_utils import cache_fn
 from tf_bind_transformer.protein_utils import get_protein_embedder
 from tf_bind_transformer.context_utils import get_text_repr, get_contextual_dim
 
+from tf_bind_transformer.attention import FeedForward, JointCrossAttention
+
 # helper functions
 
 def exists(val):
@@ -27,6 +30,8 @@ def exists(val):
 def default(val, d):
     return val if exists(val) else d
 
+def identity(fn, *args, **kwargs):
+    return fn
 
 @contextmanager
 def null_context():
@@ -288,7 +293,8 @@ class AdapterModel(nn.Module):
         else:
             self.enformer.eval()
             enformer_context = torch.no_grad()
-            enformer_forward = cache_enformer_forward(enformer_forward)
+            enformer_forward_wrapper = cache_enformer_forward if self.training else identity
+            enformer_forward = enformer_forward_wrapper(enformer_forward)
 
         # genetic sequence embedding
 
@@ -475,12 +481,7 @@ class AttentionAdapterModel(nn.Module):
 
         self.attn_dropout = nn.Dropout(dropout)
 
-        self.feedforward = nn.Sequential(
-            nn.LayerNorm(enformer_dim),
-            nn.Linear(enformer_dim, enformer_dim * 2),
-            nn.GELU(),
-            nn.Linear(enformer_dim * 2, enformer_dim)
-        )
+        self.feedforward = FeedForward(enformer_dim, dropout = dropout)
 
         # to predictions
 
@@ -549,7 +550,8 @@ class AttentionAdapterModel(nn.Module):
         else:
             self.enformer.eval()
             enformer_context = torch.no_grad()
-            enformer_forward = cache_enformer_forward(enformer_forward)
+            enformer_forward_wrapper = cache_enformer_forward if self.training else identity
+            enformer_forward = enformer_forward_wrapper(enformer_forward)
 
         # genetic sequence embedding
 
